
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% combine_design_matrices.m
%
% Purpose: combine BrainVoyager design matrix files (*.sdm, fileversion 1),
% where the first design matrix is PPI and the second a motion correction
% file (*_3DMC.sdm or *_3DMC_zscore.sdm).
%
% The first *.sdm file can have a flexible number of predictors.
% The second is supposed to be the 3D motion correction file, with the
% following predictors: 3 for translation and 3 for rotation in x, y and z-direction
% (and perhaps a constant predictor).
%
% For activation of script: load in Matlab (recent version), press 'run'.
% The script will ask for the *.sdm files by pop-up dialogs.
%
% Output: combined design matrix file (*.sdm) in directory of first *.sdm file.
%
% HB, Brain Innovation, June '16
% v0.1
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function combine_design_matrices()

[FileName,PathName] = uigetfile('*.sdm','Please select the first *.sdm file...');
sdmFileName = [PathName FileName]
sdm1 = getDesignMatrix(sdmFileName);
[FileName,PathName] = uigetfile('*.sdm','Please select the motion estimation *.sdm file...');
sdmFileName2 = [PathName FileName];
sdm2 = getDesignMatrix(sdmFileName2);

% check if same number of timepoints
if (sdm1.noOfTimePoints ~= sdm2.noOfTimePoints)
    disp('Number of time points not similar. Cannot combine design matrices')
else
    % combine
    sdm3.noOfTimePoints = sdm1.noOfTimePoints;
    sdm3.noOfPredictors = 0;
    if ((sdm1.firstConfound == sdm1.noOfPredictors) && (sdm1.includesConstant))
        sdm3.noOfPredictors = (sdm1.noOfPredictors - 1);
        sdm3.predictors = sdm1.predictors(:,1:sdm1.noOfPredictors-1)
        sdm3.names = sdm1.names(1:sdm1.noOfPredictors-1)
        sdm3.colours = sdm1.colours(1:(sdm1.noOfPredictors*3-3))
    elseif (~sdm1.includesConstant)
        sdm3.noOfPredictors = sdm1.noOfPredictors;
        sdm3.predictors = sdm1.predictors
        sdm3.names = sdm1.names;
        sdm3.colours = sdm1.colours;
    end
    if (sdm2.includesConstant)
        sdm3.noOfPredictors = sdm3.noOfPredictors + (sdm2.noOfPredictors - 1);
        sdm3.predictors = [sdm3.predictors sdm2.predictors(:,1:sdm2.noOfPredictors - 1)]
        sdm3.names = [sdm3.names; sdm2.names(1:sdm2.noOfPredictors - 1)]
        sdm3.colours = [sdm3.colours sdm2.colours(1:(sdm2.noOfPredictors*3-3))]
    else
        sdm3.noOfPredictors = sdm3.noOfPredictors + sdm2.noOfPredictors
        sdm3.predictors = [sdm3.predictors sdm2.predictors]
        sdm3.names = [sdm3.names; sdm2.names]
        sdm3.colours = [sdm3.colours sdm2.colours]
    end
    % now add a constant
    constant = ones(sdm1.noOfTimePoints,1);
    sdm3.predictors =  [sdm3.predictors constant]
    sdm3.noOfPredictors = sdm3.noOfPredictors + 1;
    sdm3.includesConstant = 1;
    sdm3.names = [sdm3.names; '"Constant"'] 
    sdm3.colours = [sdm3.colours 128 128 128]
    
    % set first confound
    choice = questdlg('Is only the first predictor of interest? (for PPI)','Yes','No')
    if strcmp(choice,'Yes')
        sdm3.firstConfound = 2;
    else
        sdm3.firstConfound = sdm1.firstConfound;
    end
    
    % write to file
    [srcPath,sourcePrt,srcExt] = fileparts(sdmFileName); % only 3 output values since 2013
    [srcPath2,sourcePrt2,srcExt2] = fileparts(sdmFileName2);
    sdmName3 = fullfile(srcPath, [sourcePrt '_' sourcePrt2 '.sdm']);
    fid = fopen(sdmName3,'wt');
    fprintf(fid, '%s\t%s\n\n', 'FileVersion:', num2str(1));
    fprintf(fid, '%s\t\t%d\n', 'NrOfPredictors:', sdm3.noOfPredictors);
    fprintf(fid, '%s\t\t%d\n', 'NrOfDataPoints:', sdm3.noOfTimePoints);
    fprintf(fid, '%s\t%d\n', 'IncludesConstant:', sdm3.includesConstant);
    fprintf(fid, '%s\t%d\n\n', 'FirstConfoundPredictor:', sdm3.firstConfound);
    % print colours
    counter=1;
    for i=1:sdm3.noOfPredictors
        for j=1:3
            fprintf(fid, '%d ', sdm3.colours(counter));
            counter = counter + 1;
        end
        fprintf(fid, '\t');
    end
    fprintf(fid, '\n');
    % print names
    for i=1:sdm3.noOfPredictors
        fprintf(fid, '%s\t', sdm3.names{i});
    end
    fprintf(fid, '\n');
    % predicted values
    for j=1:sdm3.noOfTimePoints
        for i=1:sdm3.noOfPredictors
            fprintf(fid, '\t%2.6f', sdm3.predictors{i}(j));
        end
        fprintf(fid, '\n');
    end
    fclose(fid);
    msgbox('sdm script finished. ','Finished *.sdm combination script');
end
end

function sdm = getDesignMatrix(sdmFileName)
% reads SDM file and returns SDM structure

sdmFile = textread(sdmFileName, '%s', 'delimiter','\n','whitespace','');
predLoc = strmatch('NrOfPredictors:', sdmFile);
predLine = sdmFile{predLoc};
colonLoc = strfind(predLine,':');
noOfPredictors = str2num(predLine(colonLoc+1:length(predLine)))

% check the constant
constLoc = strmatch('IncludesConstant:', sdmFile);
constLine = sdmFile{constLoc};
colon2Loc = strfind(constLine,':');
includesConstant = str2num(constLine(colon2Loc+1:length(constLine)))

% check the first confound
confLoc = strmatch('FirstConfoundPredictor:', sdmFile);
confLine = sdmFile{confLoc};
colon3Loc = strfind(confLine,':');
firstConfound = str2num(confLine(colon3Loc+1:length(confLine)))

% check the number of timepoints
dpLoc = strmatch('NrOfDataPoints:', sdmFile);
dpLine = sdmFile{dpLoc};
colon4Loc = strfind(dpLine,':');
timepoints = str2num(dpLine(colon4Loc+1:length(dpLine)))

% read the predictors
predFormatLine = ' '
for i=1:noOfPredictors
    predFormatLine = [predFormatLine ' %f ']
end
fid = fopen(sdmFileName);
sdmPreds2 = textscan(fid,predFormatLine,'headerlines', 9)
fclose(fid);

sdm.filename = sdmFileName
sdm.noOfPredictors=noOfPredictors
sdm.includesConstant=includesConstant
sdm.firstConfound=firstConfound
sdm.colours = getColours(sdmFile{8}, sdm.noOfPredictors)
sdm.names = getNames(sdmFile{9}, sdm.noOfPredictors)
sdm.predictors=sdmPreds2
sdm.noOfTimePoints = timepoints
end


function namesArray = getNames(line, noOfPredictors)

namesArray=cell(noOfPredictors,1)
counter=1;
quotepos = strfind(line,'"')
for i=1:2:(noOfPredictors*2)
    disp([num2str(i) ' : ' num2str(i+1)])
    namesArray{counter,:} = line(quotepos(i):quotepos(i+1))
    counter = counter+1;
end
disp(namesArray)
end

function colourArray = getColours(line, noOfPredictors)

formatStr = ''
for i=1:noOfPredictors
    formatStr = [formatStr '%d %d %d']
end
colourArray=cell2mat(textscan(line,formatStr))
end